/*
 * ffsox.h
 * Copyright (C) 2010, 2012 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __FFSOX_H__
#define __FFSOX_H__

#include <pb.h>

#if defined (__FFSOX_PROTO__)
#include <lame/lame.h>
#include <libavutil/opt.h>
#include <libavformat/avformat.h>
#else
#define avpicture_deinterlace DEPRECATED_avpicture_deinterlace
#include "ffsox_sym.h"
#undef avpicture_deinterlace
int avpicture_deinterlace(AVPicture *dst, const AVPicture *src,
                          enum AVPixelFormat pix_fmt, int width, int height);
#endif

#define FFSOX_VST

#define FFSOX_ZERO(p) \
  (memset(p,0,sizeof *(p)))

///////////////////////////////////////////////////////////////////////////////
typedef int16_t ffsox_s16_t;
typedef int32_t ffsox_s32_t;
typedef float ffsox_f32_t;
typedef double ffsox_f64_t;

///////////////////////////////////////////////////////////////////////////////
#if defined (FFO_CLONE_STREAM_META) // {
#define ffi_metadata(ffi) ( \
  NULL!=(ffi)->ic->metadata?(ffi)->ic->metadata: \
  NULL!=(ffi)->ast?(ffi)->ast->st->metadata: \
  NULL \
)
#else // } {
#define ffi_metadata(ffi) ( \
  NULL!=(ffi)->ic->metadata?(ffi)->ic->metadata: \
  NULL \
)
#endif // }

enum {
  FFI_CSV=1<<0,
#if ! defined (FFO_CLONE_STREAM_META) // {
  FFI_STREAM_META=1<<1,
#endif // }
  FFI_OPEN=1<<2,
  FFI_STEREO=1<<3,
  FFI_DEFAULT=FFI_CSV|FFI_STREAM_META|FFI_OPEN|FFI_STEREO
};

typedef struct ffi_st {
  AVStream *st;
  AVCodec *dec;
  AVCodecContext *dc;
  int index;
  ////
  int decoding_needed;
  int saw_first_ts;
  int64_t dts;
  int64_t next_dts;
  ////
} ffi_st_t;

ffi_st_t *ffi_st_init(ffi_st_t *ffi_st, AVStream *st, int mode);
ffi_st_t *ffi_st_close(ffi_st_t *ffi_st);

typedef struct ffi {
  AVFormatContext *ic;
  int64_t ts_offset;
  int nb_streams;
  ffi_st_t *st;
  ffi_st_t *ast;
#if defined (FFSOX_VST) // {
  ffi_st_t *vst;
#endif // }
} ffi_t;

ffi_t *ffi_init(ffi_t *ffi, pb_str_t *fn, int mode);
ffi_t *ffi_cleanup(ffi_t *ffi);

///////////////////////////////////////////////////////////////////////////////
int ffsox_clone(pb_str_t *in, pb_str_t *out, int mode, char **meta);
int ffsox_lame(pb_str_t *in, pb_str_t *out, int mode, double gain,
    double quality);

///////////////////////////////////////////////////////////////////////////////
#define ffsox_ar_valid(ar) \
  (FFSOX_AR_VALID&(ar)->state)
#define ffsox_ar_frame(ar) \
  ((ar)->cbs->frame((ar)->data))
#define ffsox_ar_stream(ar) \
  ((ar)->cbs->stream((ar)->data))
#define ffsox_ar_read(ar,pkt) \
  ((ar)->cbs->read((ar)->data,pkt))
#define ffsox_ar_got_frame(ar,frame) \
  ((ar)->cbs->got_frame((ar)->data,ar,frame))

typedef struct ffsox_ar ffsox_ar_t;
typedef struct ffsox_ar_cbs ffsox_ar_cbs_t;

enum {
  FFSOX_AR_RUN=1,
  FFSOX_AR_FLUSH=FFSOX_AR_RUN<<1,
  FFSOX_AR_DONE=FFSOX_AR_FLUSH<<1,
  FFSOX_AR_ERROR=FFSOX_AR_DONE<<1,
  FFSOX_AR_VALID=FFSOX_AR_RUN|FFSOX_AR_FLUSH
};

struct ffsox_ar_cbs {
  AVFrame *(*frame)(void *data);
  AVStream *(*stream)(void *data);
  int (*read)(void *data, AVPacket *pkt);
  void (*got_frame)(void *data, ffsox_ar_t *ar, AVFrame *frame);
};

struct ffsox_ar {
  void *data;
  ffsox_ar_cbs_t *cbs;

  double q;
  int count;
  AVPacket pkt1;
  AVPacket pkt2;
  AVFrame *frame;
  int state;
};

ffsox_ar_t *ffsox_ar_init(ffsox_ar_t *ar, double q, void *data,
    ffsox_ar_cbs_t *cbs);
ffsox_ar_t *ffsox_ar_init_ffi(ffsox_ar_t *ar, double q, ffi_t *ffi);
void ffsox_ar_reset(ffsox_ar_t *ar, double q);
ffsox_ar_t *ffsox_ar_cleanup(ffsox_ar_t *ar);
void ffsox_ar_next(ffsox_ar_t *ar);

///////////////////////////////////////////////////////////////////////////////
#define ffsox_buf_avail(b) \
  ((b)->mp-(b)->wp)

typedef struct ffsox_buf {
  uint8_t *rep;
  uint8_t *wp;
  uint8_t *mp;
} ffsox_buf_t;

ffsox_buf_t *ffsox_buf_init(ffsox_buf_t *buf, size_t size);
ffsox_buf_t *ffsox_buf_cleanup(ffsox_buf_t *buf);

ffsox_buf_t *ffsox_buf_realloc(ffsox_buf_t *buf, size_t len);

///////////////////////////////////////////////////////////////////////////////
sox_effect_handler_t const *ffsox_sox_reader();

#endif // __FSOX_H__
