/*
 * ffsox_ar.c
 * Copyright (C) 2011, 2012 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdio.h>
#include "ffsox.h"

///////////////////////////////////////////////////////////////////////////////
ffsox_ar_t *ffsox_ar_init(ffsox_ar_t *ar, double q, void *data,
    ffsox_ar_cbs_t *cbs)
{
  ar->data=data;
  ar->cbs=cbs;

  // allocate frame only if not provided by the client.
  if (NULL==(ar->frame=ffsox_ar_frame(ar))) {
    if (NULL==(ar->frame=av_frame_alloc()))
      return NULL;
  }

  ffsox_ar_reset(ar,q);

  return ar;
}

ffsox_ar_t *ffsox_ar_cleanup(ffsox_ar_t *ar)
{
  if (NULL!=ar->frame&&NULL==ffsox_ar_frame(ar)) {
    av_free(ar->frame);
    ar->frame=NULL;
  }

  return ar;
}

void ffsox_ar_reset(ffsox_ar_t *ar, double q)
{
  ar->q=q;
  ar->count=0;
  ar->state=FFSOX_AR_RUN;
  FFSOX_ZERO(&ar->pkt1);
  FFSOX_ZERO(&ar->pkt2);
}

void ffsox_ar_next(ffsox_ar_t *ar)
{
  AVStream *stream=ffsox_ar_stream(ar);
  int index=stream->index;
  AVCodecContext *dc=stream->codec;
  int len,got_frame;

  //avcodec_get_frame_defaults(ar->frame);

  while (FFSOX_AR_RUN==ar->state) {
    got_frame=0;

    if (ar->pkt2.size<=0) {
      av_free_packet(&ar->pkt1);
      FFSOX_ZERO(&ar->pkt1);

	  if (0!=ffsox_ar_read(ar,&ar->pkt1)) {
        FFSOX_ZERO(&ar->pkt1);
        FFSOX_ZERO(&ar->pkt2);
        ar->state=FFSOX_AR_FLUSH;
        break;
      }
      else if (ar->pkt1.stream_index!=index) {
        av_free_packet(&ar->pkt1);
        FFSOX_ZERO(&ar->pkt1);
        continue;
      }
	  else
	    ar->pkt2=ar->pkt1;
    }

    if ((len=avcodec_decode_audio4(dc,ar->frame,&got_frame,&ar->pkt2))<0) {
      //ar->state=FFSOX_AR_ERROR;
      //goto cleanup;
	  // ignore and continue
      FFSOX_ZERO(&ar->pkt2);
	  continue;
    }
  cleanup:
	ar->pkt2.size-=len;
	ar->pkt2.data+=len;

    if (got_frame) {
	  ffsox_ar_got_frame(ar,ar->frame);
	  //av_frame_unref(ar->frame);
	  ++ar->count;
      return;
    }
  }

  if (FFSOX_AR_FLUSH==ar->state) {
    if (avcodec_decode_audio4(dc,ar->frame,&got_frame,&ar->pkt2)<0)
      ar->state=FFSOX_AR_ERROR;
    else if (got_frame) {
	  ffsox_ar_got_frame(ar,ar->frame);
	  //av_frame_unref(ar->frame);
	  ++ar->count;
	}
	else
      ar->state=FFSOX_AR_DONE;
  }
}
