#ifndef __PB_H__
#define __PB_H__

#if defined (WIN32)
#  if ! defined (UNICODE)
#    define UNICODE
#  endif
#  if defined (UNICODE) && ! defined (_UNICODE)
#    define _UNICODE
#  endif
#  include <windows.h>
#  include <tchar.h>
#  define _PS(s)		(pb_str_t[]){{.utf8=s,.oem=s,.tcs=_T(s)}}
#else
#  define TCHAR         char
#  define _T(s)         s
#  define _PS(s)		(pb_str_t[]){{.utf8=s,.oem=s}}
#endif

#define PS_UTF8(s)      (NULL==(s)?NULL:(s)->utf8)
#define PS_OEM(s)       (NULL==(s)?NULL:(s)->oem)

#if defined (WIN32)
#  define PS_TCS(s)     (NULL==(s)?NULL:(s)->tcs)
#  define PS_SEP		_PS("\\")
#else
#  define PS_TCS(s)     (NULL==(s)?NULL:(s)->utf8)
#  define PS_SEP		_PS("/")
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

///////////////////////////////////////////////////////////////////////////////
struct AVDictionary;

typedef int (* pb_dict_set_t)(struct AVDictionary **pm, const char *key,
    const char *value, int flags);

int pb_csv2avdict(char *file, char sep, pb_dict_set_t dict_set,
    struct AVDictionary **pm);

///////////////////////////////////////////////////////////////////////////////
#if defined (WIN32)
int pb_s2ws_size(const char *s, int cp);
int pb_s2ws_copy(wchar_t *ws, const char *s, int cp, int n);
wchar_t **pb_s2ws_v(char **s, int cp);

int pb_ws2s_size(const wchar_t *ws, int cp);
int pb_ws2s_copy(char *s, const wchar_t *ws, int cp, int n);
#endif

///////////////////////////////////////////////////////////////////////////////
char *pb_name(char *s);
char *pb_ext(char *s);
#if defined (WIN32)
wchar_t *pb_wname(wchar_t *ws);
wchar_t *pb_wext(wchar_t *ws);
#endif

///////////////////////////////////////////////////////////////////////////////
typedef struct pb_str {
  char *utf8;
  char *oem;
#if defined (WIN32)
  TCHAR *tcs;
#endif
} pb_str_t;

pb_str_t *pb_str_cre(const TCHAR *ts);
pb_str_t *pb_str_cre_cs(const char *cs);
pb_str_t *pb_str_cat(TCHAR *ts, ...);
pb_str_t *pb_str_cat_ps(pb_str_t *ps, ...);
pb_str_t **pb_str_cre_n(TCHAR **ts, int n);
pb_str_t **pb_str_cre_n_ps(pb_str_t **ts, int n);
pb_str_t **pb_str_cre_v(TCHAR **ts);
pb_str_t **pb_str_cre_v_cs(char **s);
pb_str_t *pb_str_dup(pb_str_t *ps);
pb_str_t *pb_str_dup_pfx(pb_str_t *pfx, pb_str_t *ts);
pb_str_t **pb_str_dup_v(pb_str_t **ps);

pb_str_t *pb_str_home(int root);
pb_str_t **pb_str_readdir(pb_str_t *dir, int cat, int filt);
pb_str_t **pb_str_args2pargs(int argc, char **argv, int *pargc,
  pb_str_t ***pargv);
pb_str_t *pb_str_name(pb_str_t *dst, pb_str_t *src);
pb_str_t *pb_str_ext(pb_str_t *dst, pb_str_t *src);

int pb_str_stat(const pb_str_t *path, struct stat *buf);

///////////////////////////////////////////////////////////////////////////////
typedef struct pb_file pb_file_t;

struct pb_file {
  pb_file_t *prev;
  pb_file_t *next;
  pb_str_t *path;
  pb_str_t name;
  pb_str_t ext;
  pb_str_t *dn;
  pb_str_t *bn;
};

pb_file_t *pb_file_cre(pb_str_t *path);
int pb_file_test(pb_file_t *head, pb_str_t *path);
pb_file_t *pb_file_push(pb_file_t **head, pb_file_t **tail, pb_file_t *f);
pb_file_t *pb_file_pop(pb_file_t **head, pb_file_t **tail);
void pb_file_close(pb_file_t **head, pb_file_t **tail);

///////////////////////////////////////////////////////////////////////////////
typedef struct pb_path pb_path_t;

struct pb_path {
  pb_path_t *prev;
  pb_str_t *path;
};

pb_path_t *pb_path_push(pb_path_t **head, pb_path_t **tail, pb_str_t *file);
void pb_path_pop(pb_path_t **head, pb_path_t **tail);

///////////////////////////////////////////////////////////////////////////////
typedef struct pb_sym {
  char *sym;
  void *ptr;
} pb_sym_t;

int pb_load(pb_str_t *dir, pb_str_t *pfx, pb_str_t *bn, pb_str_t *ver,
    pb_sym_t *sym);

#endif // __PB_H__
